import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Scanner;

public class Grille {
    private final int taille;
    private int[][] grille;

    private Grille(int taille) {
        this.taille = taille;
        this.grille = new int[taille][taille];
    }

    public Grille(int taille,int taux){
        this(taille);
        randomGrille(taux);
    }

    public Grille(int taille, File fichier) throws IOException {
        this(taille);
        initGrille(fichier);
    }

    private void initGrille(File fichier) throws IOException {
        //on récupère le contenu du fichier
        Scanner lecteur = new java.util.Scanner(fichier);
        if(lecteur.next().equals("#Life") && lecteur.next().equals("1.06"))
            while (lecteur.hasNextInt()) {
                int i = lecteur.nextInt();
                int j = lecteur.nextInt();
                if(i>=0 && i<taille && j>=0 && j<taille) grille[i][j] = 1;
            }

        else
            System.out.println("La première ligne du fichier n'est pas correctement écrite.");
    }

    private void randomGrille(int taux) {
        for(int i=0; i<taille;i++)
            for(int j=0; j<taille;j++)
                if(Math.random()*100 < taux)
                    grille[i][j] = 1;
                else
                    grille[i][j] = 0;
    }

    public void next() {
        int[][] tmp = new int[taille][taille];

        for(int lig=0; lig<taille; lig++)
            for(int col=0; col<taille; col++) {
                //naissance
                if(grille[lig][col] == 0 && getNbVoisin(lig, col) == 3) tmp[lig][col] = 1;
                //survie
                if(grille[lig][col] == 1 && getNbVoisin(lig, col) == 2 || getNbVoisin(lig, col) == 3) tmp[lig][col] = 1;
                //mort
                if(grille[lig][col] == 1 && getNbVoisin(lig, col)  < 2 && getNbVoisin(lig, col)  > 3) tmp[lig][col] = 0;
            }

        grille = tmp;
    }

    private int getNbVoisin(int lig, int col) {
        int ligDessus = (lig-1+taille)%taille;
        int ligDessous = (lig+1+taille)%taille;
        int colGauche = (col-1+taille)%taille;
        int colDroite = (col+1+taille)%taille;

        return  grille[ligDessus][colGauche]  + grille[ligDessus][col]  + grille[ligDessus][colDroite] +
                grille[lig][colGauche]        +                           grille[lig][colDroite]       +
                grille[ligDessous][colGauche] + grille[ligDessous][col] + grille[ligDessous][colDroite];
    }

    public int getTaille() { return taille; }
    public int getCase(int lig, int col){ return grille[lig][col]; }

    public int tauxOccupation() {
        int nbVivant = 0;
        for(int lig=0; lig<taille; lig++)
            for(int col=0; col<taille; col++){
                nbVivant += grille[lig][col];
            }
        return (int)((100.0*nbVivant)/(taille*taille));
    }

    public void swap(int lig, int col){
        grille[lig][col] = (grille[lig][col] + 1)%2;
    }

    @Override
    public String toString() {
        return "Grille{" +
                "grille=" + Arrays.toString(grille) +
                '}';
    }
}
